@@ -11,11 +11,13 @@
 import math
 from collections import defaultdict
 from collections.abc import Sequence
-from typing import TYPE_CHECKING, Callable, Optional, TypeVar, Union
+from typing import TYPE_CHECKING, Callable, Optional, Union, cast
 
 import attr
 
 from hypothesis.internal.conjecture.choice import (
+    ChoiceNode,
+    ChoiceT,
     choice_equal,
     choice_from_index,
     choice_key,
@@ -25,8 +27,9 @@ from hypothesis.internal.conjecture.choice import (
 from hypothesis.internal.conjecture.data import (
     ConjectureData,
     ConjectureResult,
-    IRNode,
+    Examples,
     Status,
+    _Overrun,
     draw_choice,
 )
 from hypothesis.internal.conjecture.junkdrawer import (
@@ -51,13 +54,14 @@ from hypothesis.internal.floats import MAX_PRECISE_INTEGER
 
 if TYPE_CHECKING:
     from random import Random
+    from typing import TypeAlias
 
     from hypothesis.internal.conjecture.engine import ConjectureRunner
 
-SortKeyT = TypeVar("SortKeyT", str, bytes)
+ShrinkPredicateT: "TypeAlias" = Callable[[Union[ConjectureResult, _Overrun]], bool]
 
 
-def sort_key(nodes: Sequence[IRNode]) -> tuple[int, tuple[int, ...]]:
+def sort_key(nodes: Sequence[ChoiceNode]) -> tuple[int, tuple[int, ...]]:
     """Returns a sort key such that "simpler" choice sequences are smaller than
     "more complicated" ones.
 
@@ -103,10 +107,10 @@ class ShrinkPassDefinition:
     run_with_chooser = attr.ib()
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.run_with_chooser.__name__
 
-    def __attrs_post_init__(self):
+    def __attrs_post_init__(self) -> None:
         assert self.name not in SHRINK_PASS_DEFINITIONS, self.name
         SHRINK_PASS_DEFINITIONS[self.name] = self
 
@@ -283,7 +287,7 @@ class Shrinker:
         self,
         engine: "ConjectureRunner",
         initial: Union[ConjectureData, ConjectureResult],
-        predicate: Optional[Callable[[ConjectureData], bool]],
+        predicate: Optional[ShrinkPredicateT],
         *,
         allow_transition: Optional[
             Callable[[Union[ConjectureData, ConjectureResult], ConjectureData], bool]
@@ -327,16 +331,7 @@ class Shrinker:
 
         # Because the shrinker is also used to `pareto_optimise` in the target phase,
         # we sometimes want to allow extending buffers instead of aborting at the end.
-        if in_target_phase:  # pragma: no cover
-            # TODO_IR: this is no longer used, but it should be. See
-            # https://github.com/HypothesisWorks/hypothesis/commit/91c63bb76c970effd6cf3c013d8ed98788cf0527
-            # we'll need to adjust for the new notion of size in terms of nodes,
-            # and change self.cached_test_function_ir.
-            from hypothesis.internal.conjecture.engine import BUFFER_SIZE
-
-            self.__extend = BUFFER_SIZE
-        else:
-            self.__extend = 0
+        self.__extend = "full" if in_target_phase else 0
         self.should_explain = explain
 
     @derived_value  # type: ignore
@@ -388,29 +383,31 @@ class Shrinker:
         if self.calls - self.calls_at_last_shrink >= self.max_stall:
             raise StopShrinking
 
-    def cached_test_function_ir(self, nodes):
+    def cached_test_function(self, nodes):
         # sometimes our shrinking passes try obviously invalid things. We handle
         # discarding them in one place here.
         for node in nodes:
             if not choice_permitted(node.value, node.kwargs):
                 return None
 
-        result = self.engine.cached_test_function_ir([n.value for n in nodes])
+        result = self.engine.cached_test_function(
+            [n.value for n in nodes], extend=self.__extend
+        )
         self.incorporate_test_data(result)
         self.check_calls()
         return result
 
-    def consider_new_tree(self, tree: Sequence[IRNode]) -> bool:
-        tree = tree[: len(self.nodes)]
+    def consider_new_nodes(self, nodes: Sequence[ChoiceNode]) -> bool:
+        nodes = nodes[: len(self.nodes)]
 
-        if startswith(tree, self.nodes):
+        if startswith(nodes, self.nodes):
             return True
 
-        if sort_key(self.nodes) < sort_key(tree):
+        if sort_key(self.nodes) < sort_key(nodes):
             return False
 
         previous = self.shrink_target
-        self.cached_test_function_ir(tree)
+        self.cached_test_function(nodes)
         return previous is not self.shrink_target
 
     def incorporate_test_data(self, data):
@@ -432,7 +429,7 @@ class Shrinker:
     def random(self) -> "Random":
         return self.engine.random
 
-    def shrink(self):
+    def shrink(self) -> None:
         """Run the full set of shrinks and update shrink_target.
 
         This method is "mostly idempotent" - calling it twice is unlikely to
@@ -488,16 +485,16 @@ class Shrinker:
                 self.debug("")
         self.explain()
 
-    def explain(self):
+    def explain(self) -> None:
 
         if not self.should_explain or not self.shrink_target.arg_slices:
             return
 
-        self.max_stall = 1e999
+        self.max_stall = 2**100
         shrink_target = self.shrink_target
         nodes = self.nodes
         choices = self.choices
-        chunks = defaultdict(list)
+        chunks: dict[tuple[int, int], list[tuple[ChoiceT, ...]]] = defaultdict(list)
 
         # Before we start running experiments, let's check for known inputs which would
         # make them redundant.  The shrinking process means that we've already tried many
@@ -536,18 +533,17 @@ class Shrinker:
                 for i in range(start, end):
                     node = nodes[i]
                     if not node.was_forced:
-                        value = draw_choice(
-                            node.ir_type, node.kwargs, random=self.random
-                        )
+                        value = draw_choice(node.type, node.kwargs, random=self.random)
                         node = node.copy(with_value=value)
                     replacement.append(node.value)
 
                 attempt = choices[:start] + tuple(replacement) + choices[end:]
-                result = self.engine.cached_test_function_ir(attempt, extend="full")
+                result = self.engine.cached_test_function(attempt, extend="full")
 
                 # Turns out this was a variable-length part, so grab the infix...
                 if result.status is Status.OVERRUN:
                     continue  # pragma: no cover  # flakily covered
+                result = cast(ConjectureResult, result)
                 if not (
                     len(attempt) == len(result.choices)
                     and endswith(result.nodes, nodes[end:])
@@ -566,10 +562,11 @@ class Shrinker:
                         choices[:start] + result.choices[start:res_end] + choices[end:]
                     )
                     chunks[(start, end)].append(result.choices[start:res_end])
-                    result = self.engine.cached_test_function_ir(attempt)
+                    result = self.engine.cached_test_function(attempt)
 
                     if result.status is Status.OVERRUN:
                         continue  # pragma: no cover  # flakily covered
+                    result = cast(ConjectureResult, result)
                 else:
                     chunks[(start, end)].append(result.choices[start:end])
 
@@ -596,7 +593,7 @@ class Shrinker:
         chunks_by_start_index = sorted(chunks.items())
         for _ in range(500):  # pragma: no branch
             # no-branch here because we don't coverage-test the abort-at-500 logic.
-            new_choices = []
+            new_choices: list[ChoiceT] = []
             prev_end = 0
             for (start, end), ls in chunks_by_start_index:
                 assert prev_end <= start < end, "these chunks must be nonoverlapping"
@@ -604,7 +601,7 @@ class Shrinker:
                 new_choices.extend(self.random.choice(ls))
                 prev_end = end
 
-            result = self.engine.cached_test_function_ir(new_choices)
+            result = self.engine.cached_test_function(new_choices)
 
             # This *can't* be a shrink because none of the components were.
             assert shrink_target is self.shrink_target
@@ -621,7 +618,7 @@ class Shrinker:
                     )
                     break
 
-    def greedy_shrink(self):
+    def greedy_shrink(self) -> None:
         """Run a full set of greedy shrinks (that is, ones that will only ever
         move to a better target) and update shrink_target appropriately.
 
@@ -638,10 +635,11 @@ class Shrinker:
                 node_program("X" * 1),
                 "pass_to_descendant",
                 "reorder_examples",
-                "minimize_duplicated_nodes",
-                "minimize_individual_nodes",
+                "minimize_duplicated_choices",
+                "minimize_individual_choices",
                 "redistribute_numeric_pairs",
                 "lower_integers_together",
+                "lower_duplicated_characters",
             ]
         )
 
@@ -683,7 +681,7 @@ class Shrinker:
             nodes = self.shrink_target.nodes
             node = nodes[i]
             if (
-                node.ir_type == "integer"
+                node.type == "integer"
                 and not node.was_forced
                 and node.value <= 10
                 and node.kwargs["min_value"] == 0
@@ -700,7 +698,7 @@ class Shrinker:
                 # shape changes, as measured by either changing the number of subsequent
                 # nodes, or changing the nodes in such a way as to cause one of the
                 # previous values to no longer be valid in its position.
-                zero_attempt = self.cached_test_function_ir(
+                zero_attempt = self.cached_test_function(
                     nodes[:i] + (nodes[i].copy(with_value=0),) + nodes[i + 1 :]
                 )
                 if (
@@ -715,7 +713,7 @@ class Shrinker:
                             zero_node = zero_attempt.nodes[j]
                             orig_node = nodes[j]
                             if (
-                                zero_node.ir_type != orig_node.ir_type
+                                zero_node.type != orig_node.type
                                 or not choice_permitted(
                                     orig_node.value, zero_node.kwargs
                                 )
@@ -733,7 +731,7 @@ class Shrinker:
         while rerandomising and attempting to repair any subsequent
         changes to the shape of the test case that this causes."""
         nodes = self.shrink_target.nodes
-        initial_attempt = self.cached_test_function_ir(
+        initial_attempt = self.cached_test_function(
             nodes[:i] + (nodes[i].copy(with_value=v),) + nodes[i + 1 :]
         )
         if initial_attempt is self.shrink_target:
@@ -743,7 +741,7 @@ class Shrinker:
         initial = self.shrink_target
         examples = self.examples_starting_at[i]
         for _ in range(3):
-            random_attempt = self.engine.cached_test_function_ir(
+            random_attempt = self.engine.cached_test_function(
                 [n.value for n in prefix], extend=len(nodes)
             )
             if random_attempt.status < Status.VALID:
@@ -753,7 +751,7 @@ class Shrinker:
                 initial_ex = initial.examples[j]
                 attempt_ex = random_attempt.examples[j]
                 contents = random_attempt.nodes[attempt_ex.start : attempt_ex.end]
-                self.consider_new_tree(nodes[:i] + contents + nodes[initial_ex.end :])
+                self.consider_new_nodes(nodes[:i] + contents + nodes[initial_ex.end :])
                 if initial is not self.shrink_target:
                     return True
         return False
@@ -859,15 +857,15 @@ class Shrinker:
             passes.sort(key=reordering.__getitem__)
 
     @property
-    def nodes(self):
+    def nodes(self) -> tuple[ChoiceNode, ...]:
         return self.shrink_target.nodes
 
     @property
-    def choices(self):
+    def choices(self) -> tuple[ChoiceT, ...]:
         return self.shrink_target.choices
 
     @property
-    def examples(self):
+    def examples(self) -> Examples:
         return self.shrink_target.examples
 
     @derived_value  # type: ignore
@@ -933,7 +931,7 @@ class Shrinker:
         assert ancestor.end >= descendant.end
         assert descendant.choice_count < ancestor.choice_count
 
-        self.consider_new_tree(
+        self.consider_new_nodes(
             self.nodes[: ancestor.start]
             + self.nodes[descendant.start : descendant.end]
             + self.nodes[ancestor.end :]
@@ -978,7 +976,7 @@ class Shrinker:
         changed = []
         for i in sorted(self.__changed_nodes):
             node = self.nodes[i]
-            if node.trivial or node.ir_type != "integer":
+            if node.trivial or node.type != "integer":
                 continue
             changed.append(node)
 
@@ -1002,7 +1000,7 @@ class Shrinker:
             )
 
         def consider(n, sign):
-            return self.consider_new_tree(
+            return self.consider_new_nodes(
                 replace_all(
                     st.nodes,
                     [
@@ -1025,7 +1023,7 @@ class Shrinker:
         self.__changed_nodes.add(i)
 
     @property
-    def __changed_nodes(self):
+    def __changed_nodes(self) -> set[int]:
         if self.__last_checked_changed_at is self.shrink_target:
             return self.__all_changed_nodes
 
@@ -1037,14 +1035,14 @@ class Shrinker:
         assert sort_key(new_target.nodes) < sort_key(prev_target.nodes)
 
         if len(prev_nodes) != len(new_nodes) or any(
-            n1.ir_type != n2.ir_type for n1, n2 in zip(prev_nodes, new_nodes)
+            n1.type != n2.type for n1, n2 in zip(prev_nodes, new_nodes)
         ):
             # should we check kwargs are equal as well?
             self.__all_changed_nodes = set()
         else:
             assert len(prev_nodes) == len(new_nodes)
             for i, (n1, n2) in enumerate(zip(prev_nodes, new_nodes)):
-                assert n1.ir_type == n2.ir_type
+                assert n1.type == n2.type
                 if not choice_equal(n1.value, n2.value):
                     self.__all_changed_nodes.add(i)
 
@@ -1090,7 +1088,7 @@ class Shrinker:
             [(node.index, node.index + 1, [node.copy(with_value=n)]) for node in nodes],
         )
 
-        attempt = self.cached_test_function_ir(initial_attempt)
+        attempt = self.cached_test_function(initial_attempt)
 
         if attempt is None:
             return False
@@ -1133,16 +1131,16 @@ class Shrinker:
             # case of this function of preserving from the right instead of
             # preserving from the left. see test_can_shrink_variable_string_draws.
 
-            (index, attempt_ir_type, attempt_kwargs, _attempt_forced) = (
+            (index, attempt_choice_type, attempt_kwargs, _attempt_forced) = (
                 attempt.misaligned_at
             )
             node = self.nodes[index]
-            if node.ir_type != attempt_ir_type:
+            if node.type != attempt_choice_type:
                 return False  # pragma: no cover
             if node.was_forced:
                 return False  # pragma: no cover
 
-            if node.ir_type in {"string", "bytes"}:
+            if node.type in {"string", "bytes"}:
                 # if the size *increased*, we would have to guess what to pad with
                 # in order to try fixing up this attempt. Just give up.
                 if node.kwargs["min_size"] <= attempt_kwargs["min_size"]:
@@ -1152,7 +1150,7 @@ class Shrinker:
                 # the size decreased in our attempt. Try again, but replace with
                 # the min_size that we would have gotten, and truncate the value
                 # to that size by removing any elements past min_size.
-                return self.consider_new_tree(
+                return self.consider_new_nodes(
                     initial_attempt[: node.index]
                     + [
                         initial_attempt[node.index].copy(
@@ -1206,7 +1204,7 @@ class Shrinker:
 
         for u, v in sorted(regions_to_delete, key=lambda x: x[1] - x[0], reverse=True):
             try_with_deleted = initial_attempt[:u] + initial_attempt[v:]
-            if self.consider_new_tree(try_with_deleted):
+            if self.consider_new_nodes(try_with_deleted):
                 return True
 
         return False
@@ -1249,21 +1247,21 @@ class Shrinker:
             for u, v in reversed(discarded):
                 del attempt[u:v]
 
-            if not self.consider_new_tree(tuple(attempt)):
+            if not self.consider_new_nodes(tuple(attempt)):
                 return False
         return True
 
     @derived_value  # type: ignore
     def duplicated_nodes(self):
-        """Returns a list of nodes grouped (ir_type, value)."""
+        """Returns a list of nodes grouped (choice_type, value)."""
         duplicates = defaultdict(list)
         for node in self.nodes:
-            duplicates[(node.ir_type, choice_key(node.value))].append(node)
+            duplicates[(node.type, choice_key(node.value))].append(node)
         return list(duplicates.values())
 
     @defines_shrink_pass()
-    def minimize_duplicated_nodes(self, chooser):
-        """Find blocks that have been duplicated in multiple places and attempt
+    def minimize_duplicated_choices(self, chooser):
+        """Find choices that have been duplicated in multiple places and attempt
         to minimize all of the duplicates simultaneously.
 
         This lets us handle cases where two values can't be shrunk
@@ -1307,8 +1305,8 @@ class Shrinker:
             # The motivation for the last condition is to avoid trying weird
             # non-shrinks where we raise one node and think we lowered another
             # (but didn't).
-            return node.ir_type in {"integer", "float"} and not (
-                node.ir_type == "float"
+            return node.type in {"integer", "float"} and not (
+                node.type == "float"
                 and (math.isnan(node.value) or abs(node.value) >= MAX_PRECISE_INTEGER)
             )
 
@@ -1344,10 +1342,10 @@ class Shrinker:
 
             # if we've increased node2 to the point that we're past max precision,
             # give up - things have become too unstable.
-            if node1.ir_type == "float" and v2 >= MAX_PRECISE_INTEGER:
+            if node1.type == "float" and v2 >= MAX_PRECISE_INTEGER:
                 return False
 
-            return self.consider_new_tree(
+            return self.consider_new_nodes(
                 self.nodes[: node1.index]
                 + (node1.copy(with_value=v1),)
                 + self.nodes[node1.index + 1 : node2.index]
@@ -1360,13 +1358,13 @@ class Shrinker:
     @defines_shrink_pass()
     def lower_integers_together(self, chooser):
         node1 = chooser.choose(
-            self.nodes, lambda n: n.ir_type == "integer" and not n.trivial
+            self.nodes, lambda n: n.type == "integer" and not n.trivial
         )
         # Search up to 3 nodes ahead, to avoid quadratic time.
         node2 = self.nodes[
             chooser.choose(
                 range(node1.index + 1, min(len(self.nodes), node1.index + 3 + 1)),
-                lambda i: self.nodes[i].ir_type == "integer"
+                lambda i: self.nodes[i].type == "integer"
                 and not self.nodes[i].was_forced,
             )
         ]
@@ -1379,7 +1377,7 @@ class Shrinker:
         shrink_towards = node1.kwargs["shrink_towards"]
 
         def consider(n):
-            return self.consider_new_tree(
+            return self.consider_new_nodes(
                 self.nodes[: node1.index]
                 + (node1.copy(with_value=node1.value - n),)
                 + self.nodes[node1.index + 1 : node2.index]
@@ -1390,10 +1388,72 @@ class Shrinker:
         find_integer(lambda n: consider(shrink_towards - n))
         find_integer(lambda n: consider(n - shrink_towards))
 
+    @defines_shrink_pass()
+    def lower_duplicated_characters(self, chooser):
+        """
+        Select two string choices no more than 4 choices apart and simultaneously
+        lower characters which appear in both strings. This helps cases where the
+        same character must appear in two strings, but the actual value of the
+        character is not relevant.
+
+        This shrinking pass currently only tries lowering *all* instances of the
+        duplicated character in both strings. So for instance, given two choices:
+
+            "bbac"
+            "abbb"
+
+        we would try lowering all five of the b characters simultaneously. This
+        may fail to shrink some cases where only certain character indices are
+        correlated, for instance if only the b at index 1 could be lowered
+        simultaneously and the rest did in fact actually have to be a `b`.
+
+        It would be nice to try shrinking that case as well, but we would need good
+        safeguards because it could get very expensive to try all combinations.
+        I expect lowering all duplicates to handle most cases in the meantime.
+        """
+        node1 = chooser.choose(
+            self.nodes, lambda n: n.type == "string" and not n.trivial
+        )
+
+        # limit search to up to 4 choices ahead, to avoid quadratic behavior
+        node2 = self.nodes[
+            chooser.choose(
+                range(node1.index + 1, min(len(self.nodes), node1.index + 1 + 4)),
+                lambda i: self.nodes[i].type == "string" and not self.nodes[i].trivial
+                # select nodes which have at least one of the same character present
+                and set(node1.value) & set(self.nodes[i].value),
+            )
+        ]
+
+        duplicated_characters = set(node1.value) & set(node2.value)
+        # deterministic ordering
+        char = chooser.choose(sorted(duplicated_characters))
+        intervals = node1.kwargs["intervals"]
+
+        def copy_node(node, n):
+            # replace all duplicate characters in each string. This might miss
+            # some shrinks compared to only replacing some, but trying all possible
+            # combinations of indices could get expensive if done without some
+            # thought.
+            return node.copy(
+                with_value=node.value.replace(char, intervals.char_in_shrink_order(n))
+            )
+
+        Integer.shrink(
+            intervals.index_from_char_in_shrink_order(char),
+            lambda n: self.consider_new_nodes(
+                self.nodes[: node1.index]
+                + (copy_node(node1, n),)
+                + self.nodes[node1.index + 1 : node2.index]
+                + (copy_node(node2, n),)
+                + self.nodes[node2.index + 1 :]
+            ),
+        )
+
     def minimize_nodes(self, nodes):
-        ir_type = nodes[0].ir_type
+        choice_type = nodes[0].type
         value = nodes[0].value
-        # unlike ir_type and value, kwargs are *not* guaranteed to be equal among all
+        # unlike choice_type and value, kwargs are *not* guaranteed to be equal among all
         # passed nodes. We arbitrarily use the kwargs of the first node. I think
         # this is unsound (= leads to us trying shrinks that could not have been
         # generated), but those get discarded at test-time, and this enables useful
@@ -1401,11 +1461,11 @@ class Shrinker:
         # same operation on both basically just works.
         kwargs = nodes[0].kwargs
         assert all(
-            node.ir_type == ir_type and choice_equal(node.value, value)
+            node.type == choice_type and choice_equal(node.value, value)
             for node in nodes
         )
 
-        if ir_type == "integer":
+        if choice_type == "integer":
             shrink_towards = kwargs["shrink_towards"]
             # try shrinking from both sides towards shrink_towards.
             # we're starting from n = abs(shrink_towards - value). Because the
@@ -1420,7 +1480,7 @@ class Shrinker:
                 abs(shrink_towards - value),
                 lambda n: self.try_shrinking_nodes(nodes, shrink_towards - n),
             )
-        elif ir_type == "float":
+        elif choice_type == "float":
             self.try_shrinking_nodes(nodes, abs(value))
             Float.shrink(
                 abs(value),
@@ -1430,18 +1490,18 @@ class Shrinker:
                 abs(value),
                 lambda val: self.try_shrinking_nodes(nodes, -val),
             )
-        elif ir_type == "boolean":
+        elif choice_type == "boolean":
             # must be True, otherwise would be trivial and not selected.
             assert value is True
             # only one thing to try: false!
             self.try_shrinking_nodes(nodes, False)
-        elif ir_type == "bytes":
+        elif choice_type == "bytes":
             Bytes.shrink(
                 value,
                 lambda val: self.try_shrinking_nodes(nodes, val),
                 min_size=kwargs["min_size"],
             )
-        elif ir_type == "string":
+        elif choice_type == "string":
             String.shrink(
                 value,
                 lambda val: self.try_shrinking_nodes(nodes, val),
@@ -1465,14 +1525,14 @@ class Shrinker:
                     node
                     if node.was_forced
                     else node.copy(
-                        with_value=choice_from_index(0, node.ir_type, node.kwargs)
+                        with_value=choice_from_index(0, node.type, node.kwargs)
                     )
                 )
                 for node in nodes[ex.start : ex.end]
             ]
         )
         suffix = nodes[ex.end :]
-        attempt = self.cached_test_function_ir(prefix + replacement + suffix)
+        attempt = self.cached_test_function(prefix + replacement + suffix)
 
         if self.shrink_target is not prev:
             return
@@ -1480,11 +1540,11 @@ class Shrinker:
         if isinstance(attempt, ConjectureResult):
             new_ex = attempt.examples[i]
             new_replacement = attempt.nodes[new_ex.start : new_ex.end]
-            self.consider_new_tree(prefix + new_replacement + suffix)
+            self.consider_new_nodes(prefix + new_replacement + suffix)
 
     @defines_shrink_pass()
-    def minimize_individual_nodes(self, chooser):
-        """Attempt to minimize each node in sequence.
+    def minimize_individual_choices(self, chooser):
+        """Attempt to minimize each choice in sequence.
 
         This is the pass that ensures that e.g. each integer we draw is a
         minimum value. So it's the part that guarantees that if we e.g. do
@@ -1494,7 +1554,7 @@ class Shrinker:
 
         then in our shrunk example, x = 10 rather than say 97.
 
-        If we are unsuccessful at minimizing a node of interest we then
+        If we are unsuccessful at minimizing a choice of interest we then
         check if that's because it's changing the size of the test case and,
         if so, we also make an attempt to delete parts of the test case to
         see if that fixes it.
@@ -1521,7 +1581,7 @@ class Shrinker:
         # the size of the generated input, we'll try deleting things after that
         # node and see if the resulting attempt works.
 
-        if node.ir_type != "integer":
+        if node.type != "integer":
             # Only try this fixup logic on integer draws. Almost all size
             # dependencies are on integer draws, and if it's not, it's doing
             # something convoluted enough that it is unlikely to shrink well anyway.
@@ -1536,7 +1596,7 @@ class Shrinker:
             + (node.copy(with_value=node.value - 1),)
             + self.nodes[node.index + 1 :]
         )
-        attempt = self.cached_test_function_ir(lowered)
+        attempt = self.cached_test_function(lowered)
         if (
             attempt is None
             or attempt.status < Status.VALID
@@ -1578,10 +1638,10 @@ class Shrinker:
                     lambda i: self.examples[i].choice_count > 0,
                 )
             ]
-            self.consider_new_tree(lowered[: ex.start] + lowered[ex.end :])
+            self.consider_new_nodes(lowered[: ex.start] + lowered[ex.end :])
         else:
             node = self.nodes[chooser.choose(range(node.index + 1, len(self.nodes)))]
-            self.consider_new_tree(lowered[: node.index] + lowered[node.index + 1 :])
+            self.consider_new_nodes(lowered[: node.index] + lowered[node.index + 1 :])
 
     @defines_shrink_pass()
     def reorder_examples(self, chooser):
@@ -1614,7 +1674,7 @@ class Shrinker:
 
         Ordering.shrink(
             range(len(examples)),
-            lambda indices: self.consider_new_tree(
+            lambda indices: self.consider_new_nodes(
                 replace_all(
                     st.nodes,
                     [
@@ -1659,7 +1719,7 @@ class Shrinker:
                 else:
                     raise NotImplementedError(f"Unrecognised command {command!r}")
 
-        return self.consider_new_tree(attempt)
+        return self.consider_new_nodes(attempt)
 
 
 def shrink_pass_family(f):
